unit u_DoMD5s;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Grids, u_AncestorForm, Guage, StrUtils;

type
  TMD5Mode = (md5_Create, md5_Check);

  TDoMD5s = class(TAncestorForm)
    StringGrid: TStringGrid;
    SaveDialog: TSaveDialog;
    Panel1: TPanel;
    FileGaugeLabel: TLabel;
    BatchGaugeLabel: TLabel;
    SaveButton: TButton;
    CancelButton: TButton;
    KeyGroupBox: TGroupBox;
    WhiteImage: TImage;
    UnprocessedLabel: TLabel;
    OKLabel: TLabel;
    GreenImage: TImage;
    OrangeImage: TImage;
    RedImage: TImage;
    ErrorLabel: TLabel;
    ProcessingLabel: TLabel;
    ElapsedTimeLabel: TLabel;
    FileInfoGroupBox: TGroupBox;
    PathLabel: TLabel;
    Label1: TLabel;
    NameLabel: TLabel;
    Label2: TLabel;
    SizeLabel: TLabel;
    Label3: TLabel;
    BatchGauge: TGuage;
    FileGauge: TGuage;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
    procedure CloseButtonClick(Sender: TObject);
    procedure SaveButtonClick(Sender: TObject);
    procedure RestartButtonClick(Sender: TObject);
    procedure StringGridDrawCell(Sender: TObject; Col, Row: Integer;
      Rect: TRect; State: TGridDrawState);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormResize(Sender: TObject);
  private
  protected
    Mode: TMD5Mode;
    Root: String;
    Running: Boolean;
    Time: DWord;//Integer;
    procedure md5proc(StreamPos, StreamSize: Int64; var Cancel: Boolean);
    procedure SaveMD5File;
    procedure SaveVerify;
  public
    FileList: TStringList;  // Disregarded unless Mode = md5_Create
    MD5SumFileName: String; // Disregarded unless Mode = md5_Check
    constructor Create(AOwner: TComponent; TheMode: TMD5Mode;
      TheRoot: String); reintroduce;
    procedure FillStringGrid;
    procedure DoThoseFunkySums;
    procedure Show; reintroduce;
  end;

var
  DoMD5s: TDoMD5s;

implementation

{$R *.DFM}

uses Misc;

constructor TDoMD5s.Create(AOwner: TComponent; TheMode: TMD5Mode;
  TheRoot: String);
begin
  if (not FileExists(TheRoot)) and (not DirectoryExists(TheRoot)) then
  begin
    raise Exception.Create('Root (' + TheRoot + ') doesn''t exists!');
  end;

  Inherited Create(AOwner);

  Mode := TheMode;
  Root := TheRoot;
  Running := False;
end;

(**** Form stuff ****)

procedure TDoMD5s.FormCreate(Sender: TObject);
begin
  FileList := TStringList.Create;
  MD5SumFileName := '';

  StringGrid.Cells[0, 0] := '';
  StringGrid.Cells[1, 0] := 'File';
  StringGrid.Cells[2, 0] := 'MD5 Hash';

  if Mode = md5_Create then
    Caption := Caption + ': Generating...'
  else
    Caption := Caption + ': Checking...';
end;

procedure TDoMD5s.FormDestroy(Sender: TObject);
begin
  FileList.Free;
end;

procedure TDoMD5s.FillStringGrid;
var
  I: Integer;
  GR: TGridRect;
  ShortFileName, CRC, FileName: String;
  Warned: Boolean;
begin
  Warned := False;

  if Mode = md5_Create then
  begin
    if FileList.Count < 1 then
    begin
      raise Exception.Create('Error, Nothing to generate sums for.');
      Exit;
    end;

    for I := 0 to FileList.Count - 1 do
    begin
      ShortFileName := FileList[I];
      if copy(ShortFileName, 1, Length(Root)) <> Root then
        Continue; // Error, just skip it.
      if Root[Length(Root)] <> '\' then
        Root := Root + '\';
      Delete(ShortFileName, 1, Length(Root));
      ShortFileName := SwapChars(ShortFileName, '\', '/');

      // Don't add a line the first time or you get an empty one at the bottom.
      if StringGrid.Cells[1, 1] <> '' then
        StringGrid.RowCount := StringGrid.RowCount + 1;
      StringGrid.Cells[1, I + 1] := ShortFileName;
    end;
  end
  else // Mode = md5_Check
  begin
    if not FileExists(MD5SumFileName) then
    begin
      raise Exception.Create('Filename not valid: ' + MD5SumFileName);
      Exit;
    end;

    FileList.Clear;
    FileList.LoadFromFile(MD5SumFileName);
    if FileList.Count < 1 then
    begin
      raise Exception.Create('File to check is empty.');
      Exit;
    end;

    // Check for a rather specific case first. If the file contains only a
    // checksum, nothing else, devine which file to check from the md5 file name.
    if (FileList.Count = 1) and (Length(FileList[0]) = 32) then
    begin
      // trim the extension off the md5 file and look for a file with that name
      FileName := Copy(ExtractFileName(MD5SumFileName), 0,
        Length(ExtractFileName(MD5SumFileName)) - Length(ExtractFileExt(MD5SumFileName)));
      FileList[0] := FileList[0] + ' *' + FileName;
    end;

    for I := 0 to FileList.Count - 1 do
    begin
      Trim(FileList[I]);
      if (FileList[I] = '') or (Pos(' ', FileList[I]) = 0)
        or (FileList[I][1] = '#') or (FileList[I][1] = ';') then
        Continue; // Skip it.
      CRC := Trim(lowercase(copy(FileList[I], 1, Pos(' ', FileList[I]) - 1)));
      FileName := copy(FileList[I], Pos(' ', FileList[I]) + 1, Length(FileList[I]));
      if FileName[1] = '*' then
        Delete(FileName, 1, 1)
      else if FileName[1] = ' ' then
      begin
        Delete(FileName, 1, 1);
        if not Warned then
        begin
          Warned := True;
          if MessageDlg('The checksum file you have selected contains one or more '
            + 'ASCII generated sums (For more info, visit http://www.md5summer.'
            + 'org/ascii/).' + #13#13
            + 'Ignore and continue?', mtWarning, [mbOk, mbCancel], 0) <> mrOk then
          begin
            raise Exception.Create('Cancelled');
            Exit;
          end;
        end;
      end;
      FileName := SwapChars(FileName, '/', '\');

      // Don't add a line the first time or you get an empty one at the bottom.
      if StringGrid.Cells[1, 1] <> '' then
        StringGrid.RowCount := StringGrid.RowCount + 1;
      StringGrid.Cells[1, StringGrid.RowCount - 1] := FileName;
      StringGrid.Cells[2, StringGrid.RowCount - 1] := CRC;
    end;
  end;

  GR.Left := 0;
  GR.Top := 0;
  GR.Right := 0;
  GR.Bottom := 0;

  StringGrid.Selection := GR;
end;

procedure TDoMD5s.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if CancelButton.Caption = '&Cancel' then
  begin
    CanClose := False;
    CancelButtonClick(Nil);
  end;
end;

procedure TDoMD5s.DoThoseFunkySums;
var
  FS: TFileStream;
  I, Errors, ErrRow: Integer;
  FileName, NewSum, DlgText: String;
  TempRow: TStrings;

  procedure Error(Msg: String);
  begin
    StringGrid.Cells[2, I] := Msg;
    StringGrid.Cells[0, I] := ' ';
    inc(Errors);
    ErrorLabel.Caption := 'Error  ('
      + inttostr(Errors) + ' so far)';
  end;
begin
  if FileList.Count < 1 then
  begin
    Exit;
  end;

  Time := GetTickCount;
  Errors := 0;
  if Mode = md5_Create then
  begin
    Running := True;
    for I := 1 to StringGrid.RowCount - 1 do
    begin
      if not Running then
        Break;

      // Skip this line if it's already been done.
      if StringGrid.Cells[0, I] <> '' then
        Continue;

      if (I <> 1) and (StringGrid.RowCount <> 1) then
        BatchGauge.Position := trunc(((I - 1) / (StringGrid.RowCount - 1)) * 100)
      else
        BatchGauge.Position := 0;
      BatchGaugeLabel.Caption := 'Batch (' + IntToStr(I)
        + ' of ' + IntToStr(StringGrid.RowCount - 1) + '):';

      if (I = StringGrid.TopRow + StringGrid.VisibleRowCount - 2)
        and ((GetWindowLong(StringGrid.Handle, GWL_STYLE) and WS_VSCROLL)
        <> 0) then
        StringGrid.TopRow := StringGrid.TopRow + 1;

      Application.ProcessMessages;

      FileName := SwapChars(StringGrid.Cells[1, I], '/', '\');
      if Root[Length(Root)] <> '\' then
        Root := Root + '\';
      FileName := Root + FileName;
      if FileExists(FileName) then
      begin
        try
          FS := TFileStream.Create(FileName, (fmOpenRead or fmShareDenyNone));
          Label1.Caption := ExtractFilePath(FileName);
          Label2.Caption := ExtractFileName(FileName);
          if FS.Size > 0 then
            Label3.Caption := Format('%n', [(FS.Size / 1024)]) + ' Kb'
          else
            Label3.Caption := 'Unknown';
          StringGrid.Cells[0, I] := '-';
          StringGrid.Cells[2, I] := lowercase(md5sum(FS, md5Proc));
          if StringGrid.Cells[2, I] = 'cancelled' then
            StringGrid.Cells[0, I] := ''
          else
            StringGrid.Cells[0, I] := '.';
          FS.Free;
        Except
          on E: Exception do
            Error(E.Message);
        end;
      end // End of: if FileExists...
      else
        Error(FileName + ' does not exists.');
      FileGauge.Position := 100;
    end; // End of: for I := 1 to StringGrid.RowCount - 1 do
    BatchGauge.Position := 100;

    if Running then
      SaveMD5File;
  end
  else // Mode = md5_Check
  begin
    Running := True;
    for I := 1 to StringGrid.RowCount - 1 do
    begin
      if not Running then
        Break;

      // Skip this line if it's already been done.
      if StringGrid.Cells[0, I] <> '' then
        Continue;

      if (I <> 1) and (StringGrid.RowCount <> 1) then
        BatchGauge.Position := trunc(((I - 1) / (StringGrid.RowCount - 1)) * 100)
      else
        BatchGauge.Position := 0;
      BatchGaugeLabel.Caption := 'Batch (' + IntToStr(I)
        + ' of ' + IntToStr(StringGrid.RowCount - 1) + '):';

      if (I = StringGrid.TopRow + StringGrid.VisibleRowCount - 2)
        and ((GetWindowLong(StringGrid.Handle, GWL_STYLE) and WS_VSCROLL)
        <> 0) then
        StringGrid.TopRow := StringGrid.TopRow + 1;

      Application.ProcessMessages;

      FileName := SwapChars(StringGrid.Cells[1, I], '/', '\');
      if Root[Length(Root)] <> '\' then
        Root := Root + '\';
      FileName := Root + FileName;
      if FileExists(FileName) then
      begin
        try
          FS := TFileStream.Create(FileName, (fmOpenRead or fmShareDenyNone));
          Label1.Caption := ExtractFilePath(FileName);
          Label2.Caption := ExtractFileName(FileName);
          if FS.Size > 0 then
            Label3.Caption := Format('%n', [(FS.Size / 1024)]) + ' Kb'
          else
            Label3.Caption := 'Unknown';
          StringGrid.Cells[0, I] := '-';
          NewSum := lowercase(md5sum(FS, md5Proc));
          if NewSum = 'cancelled' then
            StringGrid.Cells[0, I] := ''
          else if (lowercase(Trim(StringGrid.Cells[2, I])) = NewSum) then
            StringGrid.Cells[0, I] := '.'
          else
            Error('Checksum did not match.');
          FS.Free;
        Except
          on E: Exception do
            Error(E.Message);
        end;
      end // End of: if FileExists...
      else
        Error(FileName + ' does not exists.');
      FileGauge.Position := 100;
    end; // End of: for I := 1 to StringGrid.RowCount - 1 do
    BatchGauge.Position := 100;
  end; // End of: Checker

  CancelButton.Caption := '&Close';
  CancelButton.Enabled := True;
  CancelButton.ModalResult := mrOk;
  CancelButton.OnClick := CloseButtonClick;
  SaveButton.Enabled := True;
//  WaitLabel.Visible := False;

  if Errors > 0 then
  begin
    DlgText := 'There ';
    if Errors = 1 then
      DlgText := DlgText + 'was ' + IntToStr(Errors)
        + ' Error while processing.'
    else
      DlgText := DlgText + 'were ' + IntToStr(Errors)
        + ' Errors while processing.';
     DlgText := DlgText + #10 + 'Do you wish to list errors?';

    if MessageDlg(DlgText, mtConfirmation, [mbYes, mbNo], 0)
      = mrYes then
    begin
      StringGrid.TopRow := 1;
      TempRow := TStringList.Create;
      ErrRow := 1;
      for I := 1 to StringGrid.RowCount - 1 do
      begin
        if StringGrid.Cells[0, I] = ' ' then
        begin
          TempRow.Assign(StringGrid.Rows[I]);
          StringGrid.Rows[I].Assign(StringGrid.Rows[ErrRow]);
          StringGrid.Rows[ErrRow].Assign(TempRow);
          ErrRow := ErrRow + 1;
        end;
      end;
      TempRow.Free;
    end;
  end;
end;

procedure TDoMD5s.CancelButtonClick(Sender: TObject);
begin
  Running := False;
  SaveButton.OnClick := RestartButtonClick;
  SaveButton.Caption := '&Restart';
  SaveButton.Enabled := True;
  CancelButton.Enabled := False;
  BatchGauge.Position := 100;
//  WaitLabel.Visible := True;
end;

procedure TDoMD5s.CloseButtonClick(Sender: TObject);
begin
  Running := False;
end;

procedure TDoMD5s.SaveButtonClick(Sender: TObject);
begin
  if Mode = md5_Create then
    SaveMD5File
  else
  begin
    SaveVerify;
  end;
end;

procedure TDoMD5s.RestartButtonClick(Sender: TObject);
begin
  CancelButton.OnClick := CancelButtonClick;
  CancelButton.ModalResult := mrNone;
  CancelButton.Caption := '&Cancel';
  CancelButton.Enabled := True;

  SaveButton.Caption := '&Save';
  SaveButton.OnClick := SaveButtonClick;
  SaveButton.Enabled := False;

  DoThoseFunkySums;
end;

procedure TDoMD5s.StringGridDrawCell(Sender: TObject; Col, Row: Integer;
  Rect: TRect; State: TGridDrawState);

  function GetRect(Image: TImage): TRect;
  begin
    Result.Left := Rect.Left + 1;
    Result.Top := Rect.Top + 1;
    Result.Right := Result.Left + Image.Width;
    Result.Bottom := Result.Top + Image.Height;
  end;
begin
  if (Col = 0) and (Row <> 0) then
  begin
    if StringGrid.Cells[Col, Row] = '.' then
      StringGrid.Canvas.BrushCopy(GetRect(GreenImage),
        GreenImage.Picture.Bitmap, GreenImage.ClientRect, clFuchsia)
    else if StringGrid.Cells[Col, Row] = ' ' then
      StringGrid.Canvas.BrushCopy(GetRect(RedImage),
        RedImage.Picture.Bitmap, RedImage.ClientRect, clFuchsia)
    else if StringGrid.Cells[Col, Row] = '-' then
      StringGrid.Canvas.BrushCopy(GetRect(OrangeImage),
        OrangeImage.Picture.Bitmap, OrangeImage.ClientRect, clFuchsia)
    else
      StringGrid.Canvas.BrushCopy(GetRect(WhiteImage),
        WhiteImage.Picture.Bitmap, WhiteImage.ClientRect, clFuchsia)
  end;
end;

procedure TDoMD5s.Show;
begin
  Inherited Show;
  DoThoseFunkySums;
  Hide;
  ShowModal;
  Hide;
end;

(**** Private procs ****)

procedure TDoMD5s.md5proc(StreamPos, StreamSize: Int64; var Cancel: Boolean);
var
  PosInMB: Int64;
begin
  if (StreamSize < 1) then
  Begin
    if (FileGauge.BackColor <> clSilver) then
    Begin
      FileGauge.Position := 0;
      FileGauge.BackColor := clSilver;
    End;
    PosInMB := StreamPos div 8388608;
    Label3.Caption := 'Unknown ('+IntToStr(PosInMB)+'MB Done)'
  End
  else
  Begin
    if FileGauge.BackColor = clSilver then
      FileGauge.BackColor := clWhite;

    if (StreamPos <> 0) and (StreamSize <> 0) then
      FileGauge.Position := trunc(((StreamPos / 8) / StreamSize) * 100);
  End;

  ElapsedTimeLabel.Caption := SecToString((GetTickCount - Time) div 1000);
  Application.ProcessMessages;
  Cancel := Running;
end;

// Save the md5sum file.
procedure TDoMD5s.SaveMD5File;
var
  I: Integer;
  Name: String;
begin
  I := 0;
  if StringGrid.RowCount - 1 = 1 then // its a file.
  begin
    Name := Root + SwapChars(StringGrid.Cells[1, 1], '/', '\');
    Name := ExtractFileName(Name);
    if Pos('.', Name) > 0 then
      I := Pos('.', Name);
      While (PosEx('.', Name, I + 1) > 0) do
        I := PosEx('.', Name, I + 1);
      Delete(Name, I, Length(Name));
    Name := Name;// + '.md5';
  end
  else if length(Root) = 3 then // its a drive.
    Name := copy(Root, 1, 1) + '_drive'//.md5'
  else // its a dir.
  begin
    Name := Root;
    if copy(Name, Length(Name), 1) = '\' then
      Delete(Name, Length(Name), 1);
    while pos('\', Name) > 0 do
      Delete(Name, 1, Pos('\', Name));
    Name := Name;// + '.md5';
  end;

  SaveDialog.FileName := Name;
  SaveDialog.InitialDir := Root;

  if SaveDialog.Execute then
    with TStringList.Create do
    begin
      Clear;

      // Header comments
      Add('# MD5 checksums generated by MD5summer (http://www.md5summer.org)');
      Add('# Generated '+DateTimeToStr(Now));
      Add('');

      for I := 1 to StringGrid.RowCount - 1 do
        if StringGrid.Cells[0, I] = '.' then
        Add(Trim(StringGrid.Cells[2, I])
          + ' *' + Trim(StringGrid.Cells[1, I]));
      SaveToFile(SaveDialog.FileName);
      Free;
    end;
end;

procedure TDoMD5s.SaveVerify;
var
  I: Integer;
  Name: String;
begin
  Name := copy(MD5SumFileName, 1, Length(MD5SumFileName)
    - (Length(ExtractFileExt(MD5SumFileName)))) + '_Result.txt';

  SaveDialog.FileName := Name;
  SaveDialog.InitialDir := Root;
  SaveDialog.FilterIndex := 2;

  if SaveDialog.Execute then
    with TStringList.Create do
    begin
      Clear;
      for I := 1 to StringGrid.RowCount - 1 do
        if StringGrid.Cells[0, I] = '.' then
          Add(Trim(StringGrid.Cells[1, I])
            + ' OK')
        else
          Add(Trim(StringGrid.Cells[1, I])
            + ' ERROR: ' + Trim(StringGrid.Cells[2, I]));
      SaveToFile(SaveDialog.FileName);
      Free;
    end;
end;

procedure TDoMD5s.FormResize(Sender: TObject);
var
  Scrollbar: Integer;
begin
  BatchGauge.Left := (Panel1.Width - BatchGauge.Width) div 2;
  BatchGaugeLabel.Left := BatchGauge.Left + 8;
  FileGauge.Left := BatchGauge.Left;
  FileGaugeLabel.Left := FileGauge.Left + 8;

  KeyGroupBox.Width := BatchGauge.Left - 4;
  FileInfoGroupBox.Width := KeyGroupBox.Width;
  FileInfoGroupBox.Left := BatchGauge.Left + BatchGauge.Width + 4;

  Label1.Width := FileInfoGroupBox.Width - Label1.Left - 2;
  Label2.Width := Label1.Width;
  Label3.Width := Label1.Width;

  SaveButton.Left := BatchGauge.Left;
  CancelButton.Left := SaveButton.Left + SaveButton.Width + 4;

  Scrollbar := 0;
  if (GetWindowLong(StringGrid.Handle, GWL_STYLE) and WS_VSCROLL) <> 0 then
    Scrollbar := 13;
  StringGrid.ColWidths[0] := StringGrid.DefaultRowHeight;
  //StringGrid.ColWidths[2] := 235;
  StringGrid.ColWidths[1] := StringGrid.Width - 7 - ScrollBar
    - StringGrid.ColWidths[0] - StringGrid.ColWidths[2];
end;

end.
