unit u_Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, Menus, u_AncestorForm, Registry,
  StShlCtl, SsBase;

type
  TMain = class(TAncestorForm)
    TopLabel: TLabel;
    OpenDialog: TOpenDialog;
    ButtonPanel: TPanel;
    CreateButton: TButton;
    VerifyButton: TButton;
    AboutButton: TButton;
    ShellTreeView: TStShellTreeView;
    procedure CreateClick(Sender: TObject);
    procedure TreeViewClick(Sender: TObject);
    procedure AboutButtonClick(Sender: TObject);
    procedure VerifyButtonClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure ShellTreeViewFolderSelected(Sender: TObject;
      Folder: TStShellFolder);
  private
  public
    constructor Create(AOwner: TComponent); override;
  end;

var
  Main: TMain;

implementation

{$R *.DFM}

uses u_CreateList, u_DoMD5s, u_About;

constructor TMain.Create(AOwner: TComponent);
var
  DoMD5s: TDoMD5s;
  Reg: TRegistry;
  CreateReg: Boolean;
begin
  CreateReg := False;

  inherited;

  try
    ShellTreeView.SelectFolder(ExtractFilePath(ParamStr(0)));
  except
    on ESsInvalidFolder do
    try
      ShellTreeView.SelectFolder(ExtractFileDrive(ParamStr(0)) + '\');
    except
    end;
  end;
  TreeViewClick(Nil);

  { Check/install file associations }
  try
    Reg := TRegistry.Create;
    With Reg do
    Begin
      RootKey := HKEY_CLASSES_ROOT;
      if keyExists('MD5Summer') then
      Begin
        // Check associated with this copy of md5summer
        OpenKey('\MD5Summer\DefaultIcon', False);
        if lowerCase(ReadString('')) <> lowerCase(ParamStr(0)+',0') then
          if MessageDlg('''.md5'' files appear to be associated with a different '
            + 'copy of md5summer. Should I associate them with this one?',
            mtConfirmation, [mbYes, mbNo], 0) = mrYes then
            CreateReg := True;
      End
      else
        CreateReg := True;

      if CreateReg then
      Begin
        // Create from scratch
        OpenKey('\MD5Summer', True);
        WriteString('', 'MD5 checksum container');
        OpenKey('\MD5Summer\DefaultIcon', True);
        WriteString('', ParamStr(0)+',0');
        OpenKey('\MD5Summer\shell\open', True);
        WriteString('', 'Verify with md5summer');
        OpenKey('\MD5Summer\shell\open\command', True);
        WriteString('', ParamStr(0)+' "%1"');
      End;

      OpenKey('\.md5', True);
      if (ReadString('') <> 'MD5Summer') and (not ValueExists('MD5summer asked')) then
      Begin
        if MessageDlg('Files with the extension ''md5'' are not associated with '
          + 'the md5summer. Should I make the association?',
          mtConfirmation, [mbYes, mbNo], 0) = mrYes then
          WriteString('', 'MD5Summer')
        else
        Begin
          // Make a note not to ask this again
          WriteBool('MD5summer asked', True);
        End;
      End;
      Free;
    End;
  except
    MessageDlg('MD5summer failed to set file associations.'+#10#13
      + 'This will not effect operation however and can be ignored.'+#10#13
      + 'Most likely you don''t have permission to access the appropriate parts'
      + ' of the registry, try running the md5summer as administrator.',
      mtInformation, [mbOk], 0);
  end;

  { If we've been provided with a file to check, check it first}
  if (ParamStr(1) <> '') and FileExists(ParamStr(1)) then
  Begin
    DoMD5s := Nil;
    try
      try
        DoMD5s := TDoMD5s.Create(Self, md5_Check, ExtractFilePath(ParamStr(1)));
        DoMD5s.MD5SumFileName := ParamStr(1);
        DoMD5s.FillStringGrid;
        DoMD5s.Show;
        ShellTreeView.SelectFolder(ExtractFilePath(ParamStr(1)));
        TreeViewClick(Nil);
      except
        on E: Exception do
          MessageDlg(E.Message, mtError, [mbOk], 0);
      end;
    finally
      DoMD5s.Free;
    end;
  end;
end;

(**** Events ****)

procedure TMain.TreeViewClick(Sender: TObject);
begin
  //CreateButton.Enabled := {FileExists(TreeView.Selection)
    //or }DirectoryExists(TreeView.Selection);
  //VerifyButton.Enabled := CreateButton.Enabled;
end;

procedure TMain.CreateClick(Sender: TObject);
var
  CreateList: TCreateList;
  DoMD5s: TDoMD5s;
begin
  // Without setting these to NIL here we get a warning that
  // they may not have been initialised when we try to free them.
  CreateList := Nil;
  DoMD5s := Nil;

  try
    try
      CreateList := TCreateList.Create(Self, ShellTreeView.SelectedFolder.Path);
      Hide;
      if CreateList.Excecute then
      begin
        DoMD5s := TDoMD5s.Create(Self, md5_Create, ShellTreeView.SelectedFolder.Path);
        DoMD5s.FileList.Assign(CreateList.FileStringList);
        DoMD5s.FillStringGrid;
        DoMD5s.Show;
      end;
      Show;
    except
      on E: Exception do
        MessageDlg(E.Message, mtError, [mbOk], 0);
    end;
  finally
    CreateList.Free;
    DoMD5s.Free;
  end;
end;

procedure TMain.VerifyButtonClick(Sender: TObject);
var
  DoMD5s: TDoMD5s;
begin
  OpenDialog.InitialDir := ShellTreeView.SelectedFolder.Path;
  DoMD5s := Nil;
  if OpenDialog.Execute then
  try
    try
      DoMD5s := TDoMD5s.Create(Self, md5_Check, ShellTreeView.SelectedFolder.Path);
      DoMD5s.MD5SumFileName := OpenDialog.FileName;
      DoMD5s.FillStringGrid;
      Hide;
      DoMD5s.Show;
      Show;
    except
      on E: Exception do
        MessageDlg(E.Message, mtError, [mbOk], 0);
    end;
  finally
    DoMD5s.Free;
  end;
end;

procedure TMain.AboutButtonClick(Sender: TObject);
var
  About: TAbout;
begin
  About := TAbout.Create(Self);
  About.ShowModal;
  About.Free;
end;

procedure TMain.FormResize(Sender: TObject);
begin
  inherited;

{  ButtonPanel.Top := ClientHeight - 4 - ButtonPanel.Height;
  ButtonPanel.Left := (ClientWidth - ButtonPanel.Width) div 2;

  TreeView.Width := ClientWidth - 8;
  TreeView.Height := ClientHeight - TreeView.Top - ButtonPanel.Height - 8;
}end;

procedure TMain.ShellTreeViewFolderSelected(Sender: TObject;
  Folder: TStShellFolder);
begin
  inherited;

  CreateButton.Enabled := DirectoryExists(Folder.Path);
  VerifyButton.Enabled := CreateButton.Enabled;
end;

end.
